@@ -7,14 +7,20 @@ use diesel::{
   r2d2::{ConnectionManager, Pool},
   PgConnection,
 };
+use doku::json::{AutoComments, Formatting};
 use lemmy_api::match_websocket_operation;
-use lemmy_api_common::blocking;
+use lemmy_api_common::{blocking, check_private_instance_and_federation_enabled};
 use lemmy_api_crud::match_websocket_operation_crud;
-use lemmy_apub::activity_queue::create_activity_queue;
-use lemmy_db_queries::{get_database_url_from_env, source::secret::Secret_};
-use lemmy_db_schema::source::secret::Secret;
+use lemmy_apub_lib::activity_queue::create_activity_queue;
+use lemmy_db_schema::{get_database_url_from_env, source::secret::Secret};
 use lemmy_routes::{feeds, images, nodeinfo, webfinger};
-use lemmy_server::{api_routes, code_migrations::run_advanced_migrations, scheduled_tasks};
+use lemmy_server::{
+  api_routes,
+  code_migrations::run_advanced_migrations,
+  init_tracing,
+  root_span_builder::QuieterRootSpanBuilder,
+  scheduled_tasks,
+};
 use lemmy_utils::{
   rate_limit::{rate_limiter::RateLimiter, RateLimit},
   request::build_user_agent,
@@ -23,16 +29,30 @@ use lemmy_utils::{
 };
 use lemmy_websocket::{chat_server::ChatServer, LemmyContext};
 use reqwest::Client;
-use std::{sync::Arc, thread};
+use reqwest_middleware::ClientBuilder;
+use reqwest_tracing::TracingMiddleware;
+use std::{env, sync::Arc, thread};
 use tokio::sync::Mutex;
+use tracing_actix_web::TracingLogger;
 
 embed_migrations!();
 
 #[actix_web::main]
 async fn main() -> Result<(), LemmyError> {
-  env_logger::init();
+  let args: Vec<String> = env::args().collect();
+  if args.len() == 2 && args[1] == "--print-config-docs" {
+    let fmt = Formatting {
+      auto_comments: AutoComments::none(),
+      ..Default::default()
+    };
+    println!("{}", doku::to_json_fmt_val(&fmt, &Settings::default()));
+    return Ok(());
+  }
+
   let settings = Settings::init().expect("Couldn't initialize settings.");
 
+  init_tracing(settings.opentelemetry_url.as_deref())?;
+
   // Set up the r2d2 connection pool
   let db_url = match get_database_url_from_env() {
     Ok(url) => url,
@@ -55,7 +75,7 @@ async fn main() -> Result<(), LemmyError> {
 
   let pool2 = pool.clone();
   thread::spawn(move || {
-    scheduled_tasks::setup(pool2);
+    scheduled_tasks::setup(pool2).expect("Couldn't set up scheduled_tasks");
   });
 
   // Set up the rate limiter
@@ -77,7 +97,13 @@ async fn main() -> Result<(), LemmyError> {
     .user_agent(build_user_agent(&settings))
     .build()?;
 
-  let activity_queue = create_activity_queue();
+  let client = ClientBuilder::new(client).with(TracingMiddleware).build();
+
+  let queue_manager = create_activity_queue(client.clone());
+
+  let activity_queue = queue_manager.queue_handle().clone();
+
+  check_private_instance_and_federation_enabled(&pool, &settings).await?;
 
   let chat_server = ChatServer::startup(
     pool.clone(),
@@ -104,13 +130,14 @@ async fn main() -> Result<(), LemmyError> {
     );
     let rate_limiter = rate_limiter.clone();
     App::new()
-      .wrap(middleware::Logger::default())
+      .wrap(actix_web::middleware::Logger::default())
+      .wrap(TracingLogger::<QuieterRootSpanBuilder>::new())
       .app_data(Data::new(context))
       // The routes
       .configure(|cfg| api_routes::config(cfg, &rate_limiter))
       .configure(|cfg| lemmy_apub::http::routes::config(cfg, &settings))
       .configure(feeds::config)
-      .configure(|cfg| images::config(cfg, &rate_limiter))
+      .configure(|cfg| images::config(cfg, client.clone(), &rate_limiter))
       .configure(nodeinfo::config)
       .configure(|cfg| webfinger::config(cfg, &settings))
   })
@@ -118,5 +145,7 @@ async fn main() -> Result<(), LemmyError> {
   .run()
   .await?;
 
+  drop(queue_manager);
+
   Ok(())
 }
